@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2014-2020 Trollsift Developers
+# Copyright (c) 2014-2022 Trollsift Developers
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -39,11 +39,22 @@ class Parser(object):
         '''
         return parse(self.fmt, stri, full_match=full_match)
 
-    def compose(self, keyvals):
-        '''Return string composed according to *fmt* string and filled
-        with values with the corresponding keys in *keyvals* dictionary.
-        '''
-        return compose(self.fmt, keyvals)
+    def compose(self, keyvals, allow_partial=False):
+        """Compose format string *self.fmt* with parameters given in the *keyvals* dict.
+
+        Args:
+            keyvals (dict): "Parameter --> parameter value" map
+            allow_partial (bool): If True, then partial composition is allowed, i.e.,
+                not all parameters present in `fmt` need to be specified in `keyvals`.
+                Unspecified parameters will, in this case, be left unchanged.
+                (Default value = False).
+
+        Returns:
+            str: Result of formatting the *self.fmt* string with parameter values
+                extracted from the corresponding items in the *keyvals* dictionary.
+
+        """
+        return compose(fmt=self.fmt, keyvals=keyvals, allow_partial=allow_partial)
 
     format = compose
 
@@ -152,13 +163,14 @@ spec_regexes['e'] = spec_regexes['f']
 spec_regexes['E'] = spec_regexes['f']
 spec_regexes['g'] = spec_regexes['f']
 spec_regexes['X'] = spec_regexes['x']
-allow_multiple = ['c', 'd', 'o', 's', 'x', 'X']
+spec_regexes[''] = spec_regexes['s']
+allow_multiple = ['c', 'd', 'o', 's', '', 'x', 'X']
 fixed_point_types = ['f', 'e', 'E', 'g']
 # format_spec ::=  [[fill]align][sign][#][0][width][,][.precision][type]
 # https://docs.python.org/3.4/library/string.html#format-specification-mini-language
 fmt_spec_regex = re.compile(
     r'(?P<align>(?P<fill>.)?[<>=^])?(?P<sign>[\+\-\s])?(?P<pound>#)?(?P<zero>0)?(?P<width>\d+)?'
-    r'(?P<comma>,)?(?P<precision>.\d+)?(?P<type>[bcdeEfFgGnosxX%])')
+    r'(?P<comma>,)?(?P<precision>.\d+)?(?P<type>[bcdeEfFgGnosxX%]?)')
 
 
 def _get_fixed_point_regex(regex_dict, width, precision):
@@ -283,7 +295,7 @@ class RegexFormatter(string.Formatter):
         if fill is None:
             if width is not None and width[0] == '0':
                 fill = '0'
-            elif ftype in ['s', 'd']:
+            elif ftype in ['s', '', 'd']:
                 fill = ' '
 
         char_type = spec_regexes[ftype]
@@ -293,7 +305,7 @@ class RegexFormatter(string.Formatter):
                 width=width,
                 precision=precision
             )
-        if ftype == 's' and align and align.endswith('='):
+        if ftype in ('s', '') and align and align.endswith('='):
             raise ValueError("Invalid format specification: '{}'".format(format_spec))
         final_regex = char_type
         if ftype in allow_multiple and (not width or width == '0'):
@@ -376,19 +388,15 @@ def _get_number_from_fmt(fmt):
 
 def _convert(convdef, stri):
     """Convert the string *stri* to the given conversion definition *convdef*."""
-    is_fixed_point = any([ftype in convdef for ftype in fixed_point_types])
     if '%' in convdef:
         result = dt.datetime.strptime(stri, convdef)
-    elif 'd' in convdef or 's' in convdef or is_fixed_point:
-        stri = _strip_padding(convdef, stri)
-        if 'd' in convdef:
-            result = int(stri)
-        elif is_fixed_point:
-            result = float(stri)
-        else:
-            result = stri
     else:
-        result = stri
+        result = _strip_padding(convdef, stri)
+        if 'd' in convdef:
+            result = int(result)
+        elif any(float_type_marker in convdef for float_type_marker in fixed_point_types):
+            result = float(result)
+
     return result
 
 
@@ -446,9 +454,25 @@ def parse(fmt, stri, full_match=True):
     return keyvals
 
 
-def compose(fmt, keyvals):
-    """Convert parameters in `keyvals` to a string based on `fmt` string."""
-    return formatter.format(fmt, **keyvals)
+def compose(fmt, keyvals, allow_partial=False):
+    """Compose format string *self.fmt* with parameters given in the *keyvals* dict.
+
+    Args:
+        fmt (str): Python format string to match against
+        keyvals (dict): "Parameter --> parameter value" map
+        allow_partial (bool): If True, then partial composition is allowed, i.e.,
+            not all parameters present in `fmt` need to be specified in `keyvals`.
+            Unspecified parameters will, in this case, be left unchanged.
+            (Default value = False).
+
+    Returns:
+        str: Result of formatting the *self.fmt* string with parameter values
+            extracted from the corresponding items in the *keyvals* dictionary.
+
+    """
+    if allow_partial:
+        return _partial_compose(fmt=fmt, keyvals=keyvals)
+    return _strict_compose(fmt=fmt, keyvals=keyvals)
 
 
 DT_FMT = {
@@ -641,3 +665,51 @@ def purge():
     """
     regex_formatter.format.cache_clear()
     get_convert_dict.cache_clear()
+
+
+def _strict_compose(fmt, keyvals):
+    """Convert parameters in `keyvals` to a string based on `fmt` string."""
+    return formatter.format(fmt, **keyvals)
+
+
+def _partial_compose(fmt, keyvals):
+    """Convert parameters in `keyvals` to a string based on `fmt` string.
+
+    Similar to _strict_compose, but accepts partial composing, i.e., not all
+    parameters in `fmt` need to be specified in `keyvals`. Unspecified parameters
+    are left unchanged.
+
+    Args:
+        fmt (str): Python format string to match against
+        keyvals (dict): "Parameter --> parameter value" map
+
+    """
+    fmt, undefined_vars = _replace_undefined_params_with_placeholders(fmt, keyvals)
+    composed_string = _strict_compose(fmt=fmt, keyvals=keyvals)
+    for fmt_placeholder, fmt_specification in undefined_vars.items():
+        composed_string = composed_string.replace(fmt_placeholder, fmt_specification)
+
+    return composed_string
+
+
+def _replace_undefined_params_with_placeholders(fmt, keyvals=None):
+    """Replace with placeholders params in `fmt` not specified in `keyvals`."""
+    vars_left_undefined = get_convert_dict(fmt).keys()
+    if keyvals is not None:
+        vars_left_undefined -= keyvals.keys()
+
+    undefined_vars_placeholders_dict = {}
+    new_fmt = fmt
+    for var in sorted(vars_left_undefined):
+        matches = set(
+            match.group()
+            for match in re.finditer(rf"{{{re.escape(var)}([^\w{{}}].*?)*}}", new_fmt)
+        )
+        if len(matches) == 0:
+            raise ValueError(f"Could not capture definitions for {var} from {fmt}")
+        for var_specification in matches:
+            fmt_placeholder = f"({hex(hash(var_specification))})"
+            undefined_vars_placeholders_dict[fmt_placeholder] = var_specification
+            new_fmt = new_fmt.replace(var_specification, fmt_placeholder)
+
+    return new_fmt, undefined_vars_placeholders_dict
